// Code generated by MockGen. DO NOT EDIT.
// Source: github.com/hetznercloud/cli/internal/testutil/terminal (interfaces: Terminal)
//
// Generated by this command:
//
//	mockgen -package terminal -destination zz_terminal_mock.go . Terminal
//

// Package terminal is a generated GoMock package.
package terminal

import (
	reflect "reflect"

	gomock "go.uber.org/mock/gomock"
)

// MockTerminal is a mock of Terminal interface.
type MockTerminal struct {
	ctrl     *gomock.Controller
	recorder *MockTerminalMockRecorder
	isgomock struct{}
}

// MockTerminalMockRecorder is the mock recorder for MockTerminal.
type MockTerminalMockRecorder struct {
	mock *MockTerminal
}

// NewMockTerminal creates a new mock instance.
func NewMockTerminal(ctrl *gomock.Controller) *MockTerminal {
	mock := &MockTerminal{ctrl: ctrl}
	mock.recorder = &MockTerminalMockRecorder{mock}
	return mock
}

// EXPECT returns an object that allows the caller to indicate expected use.
func (m *MockTerminal) EXPECT() *MockTerminalMockRecorder {
	return m.recorder
}

// ReadPassword mocks base method.
func (m *MockTerminal) ReadPassword(fd int) ([]byte, error) {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "ReadPassword", fd)
	ret0, _ := ret[0].([]byte)
	ret1, _ := ret[1].(error)
	return ret0, ret1
}

// ReadPassword indicates an expected call of ReadPassword.
func (mr *MockTerminalMockRecorder) ReadPassword(fd any) *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "ReadPassword", reflect.TypeOf((*MockTerminal)(nil).ReadPassword), fd)
}

// StdoutIsTerminal mocks base method.
func (m *MockTerminal) StdoutIsTerminal() bool {
	m.ctrl.T.Helper()
	ret := m.ctrl.Call(m, "StdoutIsTerminal")
	ret0, _ := ret[0].(bool)
	return ret0
}

// StdoutIsTerminal indicates an expected call of StdoutIsTerminal.
func (mr *MockTerminalMockRecorder) StdoutIsTerminal() *gomock.Call {
	mr.mock.ctrl.T.Helper()
	return mr.mock.ctrl.RecordCallWithMethodType(mr.mock, "StdoutIsTerminal", reflect.TypeOf((*MockTerminal)(nil).StdoutIsTerminal))
}
