// Code generated by gowrap. DO NOT EDIT.
// template: schema.tmpl
// gowrap: http://github.com/hexdigest/gowrap

package hcloud

import (
	"github.com/hetznercloud/hcloud-go/v2/hcloud/schema"
)

var c converter

// ActionFromSchema converts schema.Action to *Action.
func ActionFromSchema(in schema.Action) *Action {
	return c.ActionFromSchema(in)
}

// ActionsFromSchema converts []schema.Action to []*Action.
func ActionsFromSchema(in []schema.Action) []*Action {
	return c.ActionsFromSchema(in)
}

// CertificateFromSchema converts schema.Certificate to *Certificate.
func CertificateFromSchema(in schema.Certificate) *Certificate {
	return c.CertificateFromSchema(in)
}

// DatacenterFromSchema converts schema.Datacenter to *Datacenter.
func DatacenterFromSchema(in schema.Datacenter) *Datacenter {
	return c.DatacenterFromSchema(in)
}

// DeprecationFromSchema converts *schema.DeprecationInfo to *DeprecationInfo.
func DeprecationFromSchema(in *schema.DeprecationInfo) *DeprecationInfo {
	return c.DeprecationFromSchema(in)
}

// ErrorFromSchema converts schema.Error to Error.
func ErrorFromSchema(in schema.Error) Error {
	return c.ErrorFromSchema(in)
}

// FirewallFromSchema converts schema.Firewall to *Firewall.
func FirewallFromSchema(in schema.Firewall) *Firewall {
	return c.FirewallFromSchema(in)
}

// FloatingIPFromSchema converts schema.FloatingIP to *FloatingIP.
func FloatingIPFromSchema(in schema.FloatingIP) *FloatingIP {
	return c.FloatingIPFromSchema(in)
}

// ISOFromSchema converts schema.ISO to *ISO.
func ISOFromSchema(in schema.ISO) *ISO {
	return c.ISOFromSchema(in)
}

// ImageFromSchema converts schema.Image to *Image.
func ImageFromSchema(in schema.Image) *Image {
	return c.ImageFromSchema(in)
}

// LoadBalancerFromSchema converts schema.LoadBalancer to *LoadBalancer.
func LoadBalancerFromSchema(in schema.LoadBalancer) *LoadBalancer {
	return c.LoadBalancerFromSchema(in)
}

// LoadBalancerMetricsFromSchema converts *schema.LoadBalancerGetMetricsResponse to *LoadBalancerMetrics.
func LoadBalancerMetricsFromSchema(in *schema.LoadBalancerGetMetricsResponse) (*LoadBalancerMetrics, error) {
	return c.LoadBalancerMetricsFromSchema(in)
}

// LoadBalancerServiceFromSchema converts schema.LoadBalancerService to LoadBalancerService.
func LoadBalancerServiceFromSchema(in schema.LoadBalancerService) LoadBalancerService {
	return c.LoadBalancerServiceFromSchema(in)
}

// LoadBalancerServiceHealthCheckFromSchema converts *schema.LoadBalancerServiceHealthCheck to LoadBalancerServiceHealthCheck.
func LoadBalancerServiceHealthCheckFromSchema(in *schema.LoadBalancerServiceHealthCheck) LoadBalancerServiceHealthCheck {
	return c.LoadBalancerServiceHealthCheckFromSchema(in)
}

// LoadBalancerTargetFromSchema converts schema.LoadBalancerTarget to LoadBalancerTarget.
func LoadBalancerTargetFromSchema(in schema.LoadBalancerTarget) LoadBalancerTarget {
	return c.LoadBalancerTargetFromSchema(in)
}

// LoadBalancerTargetHealthStatusFromSchema converts schema.LoadBalancerTargetHealthStatus to LoadBalancerTargetHealthStatus.
func LoadBalancerTargetHealthStatusFromSchema(in schema.LoadBalancerTargetHealthStatus) LoadBalancerTargetHealthStatus {
	return c.LoadBalancerTargetHealthStatusFromSchema(in)
}

// LoadBalancerTargetServerFromSchema converts schema.LoadBalancerTargetServer to LoadBalancerTargetServer.
func LoadBalancerTargetServerFromSchema(in schema.LoadBalancerTargetServer) LoadBalancerTargetServer {
	return c.LoadBalancerTargetServerFromSchema(in)
}

// LoadBalancerTypeFromSchema converts schema.LoadBalancerType to *LoadBalancerType.
func LoadBalancerTypeFromSchema(in schema.LoadBalancerType) *LoadBalancerType {
	return c.LoadBalancerTypeFromSchema(in)
}

// LoadBalancerTypeLocationPricingFromSchema converts schema.PricingLoadBalancerTypePrice to LoadBalancerTypeLocationPricing.
func LoadBalancerTypeLocationPricingFromSchema(in schema.PricingLoadBalancerTypePrice) LoadBalancerTypeLocationPricing {
	return c.LoadBalancerTypeLocationPricingFromSchema(in)
}

// LocationFromSchema converts schema.Location to *Location.
func LocationFromSchema(in schema.Location) *Location {
	return c.LocationFromSchema(in)
}

// NetworkFromSchema converts schema.Network to *Network.
func NetworkFromSchema(in schema.Network) *Network {
	return c.NetworkFromSchema(in)
}

// NetworkRouteFromSchema converts schema.NetworkRoute to NetworkRoute.
func NetworkRouteFromSchema(in schema.NetworkRoute) NetworkRoute {
	return c.NetworkRouteFromSchema(in)
}

// NetworkSubnetFromSchema converts schema.NetworkSubnet to NetworkSubnet.
func NetworkSubnetFromSchema(in schema.NetworkSubnet) NetworkSubnet {
	return c.NetworkSubnetFromSchema(in)
}

// PaginationFromSchema converts schema.MetaPagination to Pagination.
func PaginationFromSchema(in schema.MetaPagination) Pagination {
	return c.PaginationFromSchema(in)
}

// PlacementGroupFromSchema converts schema.PlacementGroup to *PlacementGroup.
func PlacementGroupFromSchema(in schema.PlacementGroup) *PlacementGroup {
	return c.PlacementGroupFromSchema(in)
}

// PriceFromSchema converts schema.Price to Price.
func PriceFromSchema(in schema.Price) Price {
	return c.PriceFromSchema(in)
}

// PricingFromSchema converts schema.Pricing to Pricing.
func PricingFromSchema(in schema.Pricing) Pricing {
	return c.PricingFromSchema(in)
}

// PrimaryIPFromSchema converts schema.PrimaryIP to *PrimaryIP.
func PrimaryIPFromSchema(in schema.PrimaryIP) *PrimaryIP {
	return c.PrimaryIPFromSchema(in)
}

// SSHKeyFromSchema converts schema.SSHKey to *SSHKey.
func SSHKeyFromSchema(in schema.SSHKey) *SSHKey {
	return c.SSHKeyFromSchema(in)
}

// SchemaFromAction converts *Action to schema.Action.
func SchemaFromAction(in *Action) schema.Action {
	return c.SchemaFromAction(in)
}

// SchemaFromActions converts []*Action to []schema.Action.
func SchemaFromActions(in []*Action) []schema.Action {
	return c.SchemaFromActions(in)
}

// SchemaFromCertificate converts *Certificate to schema.Certificate.
func SchemaFromCertificate(in *Certificate) schema.Certificate {
	return c.SchemaFromCertificate(in)
}

// SchemaFromDatacenter converts *Datacenter to schema.Datacenter.
func SchemaFromDatacenter(in *Datacenter) schema.Datacenter {
	return c.SchemaFromDatacenter(in)
}

// SchemaFromDeprecation converts *DeprecationInfo to *schema.DeprecationInfo.
func SchemaFromDeprecation(in *DeprecationInfo) *schema.DeprecationInfo {
	return c.SchemaFromDeprecation(in)
}

// SchemaFromError converts Error to schema.Error.
func SchemaFromError(in Error) schema.Error {
	return c.SchemaFromError(in)
}

// SchemaFromFirewall converts *Firewall to schema.Firewall.
func SchemaFromFirewall(in *Firewall) schema.Firewall {
	return c.SchemaFromFirewall(in)
}

// SchemaFromFirewallCreateOpts converts FirewallCreateOpts to schema.FirewallCreateRequest.
func SchemaFromFirewallCreateOpts(in FirewallCreateOpts) schema.FirewallCreateRequest {
	return c.SchemaFromFirewallCreateOpts(in)
}

// SchemaFromFirewallResource converts FirewallResource to schema.FirewallResource.
func SchemaFromFirewallResource(in FirewallResource) schema.FirewallResource {
	return c.SchemaFromFirewallResource(in)
}

// SchemaFromFirewallSetRulesOpts converts FirewallSetRulesOpts to schema.FirewallActionSetRulesRequest.
func SchemaFromFirewallSetRulesOpts(in FirewallSetRulesOpts) schema.FirewallActionSetRulesRequest {
	return c.SchemaFromFirewallSetRulesOpts(in)
}

// SchemaFromFloatingIP converts *FloatingIP to schema.FloatingIP.
func SchemaFromFloatingIP(in *FloatingIP) schema.FloatingIP {
	return c.SchemaFromFloatingIP(in)
}

// SchemaFromISO converts *ISO to schema.ISO.
func SchemaFromISO(in *ISO) schema.ISO {
	return c.SchemaFromISO(in)
}

// SchemaFromImage converts *Image to schema.Image.
func SchemaFromImage(in *Image) schema.Image {
	return c.SchemaFromImage(in)
}

// SchemaFromLoadBalancer converts *LoadBalancer to schema.LoadBalancer.
func SchemaFromLoadBalancer(in *LoadBalancer) schema.LoadBalancer {
	return c.SchemaFromLoadBalancer(in)
}

// SchemaFromLoadBalancerAddServiceOpts converts LoadBalancerAddServiceOpts to schema.LoadBalancerActionAddServiceRequest.
func SchemaFromLoadBalancerAddServiceOpts(in LoadBalancerAddServiceOpts) schema.LoadBalancerActionAddServiceRequest {
	return c.SchemaFromLoadBalancerAddServiceOpts(in)
}

// SchemaFromLoadBalancerCreateOpts converts LoadBalancerCreateOpts to schema.LoadBalancerCreateRequest.
func SchemaFromLoadBalancerCreateOpts(in LoadBalancerCreateOpts) schema.LoadBalancerCreateRequest {
	return c.SchemaFromLoadBalancerCreateOpts(in)
}

// SchemaFromLoadBalancerCreateOptsTargetServer converts LoadBalancerCreateOptsTargetServer to schema.LoadBalancerCreateRequestTargetServer.
func SchemaFromLoadBalancerCreateOptsTargetServer(in LoadBalancerCreateOptsTargetServer) schema.LoadBalancerCreateRequestTargetServer {
	return c.SchemaFromLoadBalancerCreateOptsTargetServer(in)
}

// SchemaFromLoadBalancerServerTarget converts LoadBalancerTargetServer to schema.LoadBalancerTargetServer.
func SchemaFromLoadBalancerServerTarget(in LoadBalancerTargetServer) schema.LoadBalancerTargetServer {
	return c.SchemaFromLoadBalancerServerTarget(in)
}

// SchemaFromLoadBalancerService converts LoadBalancerService to schema.LoadBalancerService.
func SchemaFromLoadBalancerService(in LoadBalancerService) schema.LoadBalancerService {
	return c.SchemaFromLoadBalancerService(in)
}

// SchemaFromLoadBalancerServiceHealthCheck converts LoadBalancerServiceHealthCheck to *schema.LoadBalancerServiceHealthCheck.
func SchemaFromLoadBalancerServiceHealthCheck(in LoadBalancerServiceHealthCheck) *schema.LoadBalancerServiceHealthCheck {
	return c.SchemaFromLoadBalancerServiceHealthCheck(in)
}

// SchemaFromLoadBalancerTarget converts LoadBalancerTarget to schema.LoadBalancerTarget.
func SchemaFromLoadBalancerTarget(in LoadBalancerTarget) schema.LoadBalancerTarget {
	return c.SchemaFromLoadBalancerTarget(in)
}

// SchemaFromLoadBalancerTargetHealthStatus converts LoadBalancerTargetHealthStatus to schema.LoadBalancerTargetHealthStatus.
func SchemaFromLoadBalancerTargetHealthStatus(in LoadBalancerTargetHealthStatus) schema.LoadBalancerTargetHealthStatus {
	return c.SchemaFromLoadBalancerTargetHealthStatus(in)
}

// SchemaFromLoadBalancerType converts *LoadBalancerType to schema.LoadBalancerType.
func SchemaFromLoadBalancerType(in *LoadBalancerType) schema.LoadBalancerType {
	return c.SchemaFromLoadBalancerType(in)
}

// SchemaFromLoadBalancerTypeLocationPricing converts LoadBalancerTypeLocationPricing to schema.PricingLoadBalancerTypePrice.
func SchemaFromLoadBalancerTypeLocationPricing(in LoadBalancerTypeLocationPricing) schema.PricingLoadBalancerTypePrice {
	return c.SchemaFromLoadBalancerTypeLocationPricing(in)
}

// SchemaFromLoadBalancerUpdateServiceOpts converts LoadBalancerUpdateServiceOpts to schema.LoadBalancerActionUpdateServiceRequest.
func SchemaFromLoadBalancerUpdateServiceOpts(in LoadBalancerUpdateServiceOpts) schema.LoadBalancerActionUpdateServiceRequest {
	return c.SchemaFromLoadBalancerUpdateServiceOpts(in)
}

// SchemaFromLocation converts *Location to schema.Location.
func SchemaFromLocation(in *Location) schema.Location {
	return c.SchemaFromLocation(in)
}

// SchemaFromNetwork converts *Network to schema.Network.
func SchemaFromNetwork(in *Network) schema.Network {
	return c.SchemaFromNetwork(in)
}

// SchemaFromNetworkRoute converts NetworkRoute to schema.NetworkRoute.
func SchemaFromNetworkRoute(in NetworkRoute) schema.NetworkRoute {
	return c.SchemaFromNetworkRoute(in)
}

// SchemaFromNetworkSubnet converts NetworkSubnet to schema.NetworkSubnet.
func SchemaFromNetworkSubnet(in NetworkSubnet) schema.NetworkSubnet {
	return c.SchemaFromNetworkSubnet(in)
}

// SchemaFromPagination converts Pagination to schema.MetaPagination.
func SchemaFromPagination(in Pagination) schema.MetaPagination {
	return c.SchemaFromPagination(in)
}

// SchemaFromPlacementGroup converts *PlacementGroup to schema.PlacementGroup.
func SchemaFromPlacementGroup(in *PlacementGroup) schema.PlacementGroup {
	return c.SchemaFromPlacementGroup(in)
}

// SchemaFromPlacementGroupCreateOpts converts PlacementGroupCreateOpts to schema.PlacementGroupCreateRequest.
func SchemaFromPlacementGroupCreateOpts(in PlacementGroupCreateOpts) schema.PlacementGroupCreateRequest {
	return c.SchemaFromPlacementGroupCreateOpts(in)
}

// SchemaFromPricing converts Pricing to schema.Pricing.
func SchemaFromPricing(in Pricing) schema.Pricing {
	return c.SchemaFromPricing(in)
}

// SchemaFromPrimaryIP converts *PrimaryIP to schema.PrimaryIP.
func SchemaFromPrimaryIP(in *PrimaryIP) schema.PrimaryIP {
	return c.SchemaFromPrimaryIP(in)
}

// SchemaFromPrimaryIPAssignOpts converts PrimaryIPAssignOpts to schema.PrimaryIPActionAssignRequest.
func SchemaFromPrimaryIPAssignOpts(in PrimaryIPAssignOpts) schema.PrimaryIPActionAssignRequest {
	return c.SchemaFromPrimaryIPAssignOpts(in)
}

// SchemaFromPrimaryIPChangeDNSPtrOpts converts PrimaryIPChangeDNSPtrOpts to schema.PrimaryIPActionChangeDNSPtrRequest.
func SchemaFromPrimaryIPChangeDNSPtrOpts(in PrimaryIPChangeDNSPtrOpts) schema.PrimaryIPActionChangeDNSPtrRequest {
	return c.SchemaFromPrimaryIPChangeDNSPtrOpts(in)
}

// SchemaFromPrimaryIPChangeProtectionOpts converts PrimaryIPChangeProtectionOpts to schema.PrimaryIPActionChangeProtectionRequest.
func SchemaFromPrimaryIPChangeProtectionOpts(in PrimaryIPChangeProtectionOpts) schema.PrimaryIPActionChangeProtectionRequest {
	return c.SchemaFromPrimaryIPChangeProtectionOpts(in)
}

// SchemaFromPrimaryIPCreateOpts converts PrimaryIPCreateOpts to schema.PrimaryIPCreateRequest.
func SchemaFromPrimaryIPCreateOpts(in PrimaryIPCreateOpts) schema.PrimaryIPCreateRequest {
	return c.SchemaFromPrimaryIPCreateOpts(in)
}

// SchemaFromPrimaryIPUpdateOpts converts PrimaryIPUpdateOpts to schema.PrimaryIPUpdateRequest.
func SchemaFromPrimaryIPUpdateOpts(in PrimaryIPUpdateOpts) schema.PrimaryIPUpdateRequest {
	return c.SchemaFromPrimaryIPUpdateOpts(in)
}

// SchemaFromSSHKey converts *SSHKey to schema.SSHKey.
func SchemaFromSSHKey(in *SSHKey) schema.SSHKey {
	return c.SchemaFromSSHKey(in)
}

// SchemaFromServer converts *Server to schema.Server.
func SchemaFromServer(in *Server) schema.Server {
	return c.SchemaFromServer(in)
}

// SchemaFromServerPrivateNet converts ServerPrivateNet to schema.ServerPrivateNet.
func SchemaFromServerPrivateNet(in ServerPrivateNet) schema.ServerPrivateNet {
	return c.SchemaFromServerPrivateNet(in)
}

// SchemaFromServerPublicNet converts ServerPublicNet to schema.ServerPublicNet.
func SchemaFromServerPublicNet(in ServerPublicNet) schema.ServerPublicNet {
	return c.SchemaFromServerPublicNet(in)
}

// SchemaFromServerPublicNetIPv4 converts ServerPublicNetIPv4 to schema.ServerPublicNetIPv4.
func SchemaFromServerPublicNetIPv4(in ServerPublicNetIPv4) schema.ServerPublicNetIPv4 {
	return c.SchemaFromServerPublicNetIPv4(in)
}

// SchemaFromServerPublicNetIPv6 converts ServerPublicNetIPv6 to schema.ServerPublicNetIPv6.
func SchemaFromServerPublicNetIPv6(in ServerPublicNetIPv6) schema.ServerPublicNetIPv6 {
	return c.SchemaFromServerPublicNetIPv6(in)
}

// SchemaFromServerType converts *ServerType to schema.ServerType.
func SchemaFromServerType(in *ServerType) schema.ServerType {
	return c.SchemaFromServerType(in)
}

// SchemaFromStorageBox converts *StorageBox to schema.StorageBox.
func SchemaFromStorageBox(in *StorageBox) schema.StorageBox {
	return c.SchemaFromStorageBox(in)
}

// SchemaFromStorageBoxChangeProtectionOpts converts StorageBoxChangeProtectionOpts to schema.StorageBoxChangeProtectionRequest.
func SchemaFromStorageBoxChangeProtectionOpts(in StorageBoxChangeProtectionOpts) schema.StorageBoxChangeProtectionRequest {
	return c.SchemaFromStorageBoxChangeProtectionOpts(in)
}

// SchemaFromStorageBoxChangeTypeOpts converts StorageBoxChangeTypeOpts to schema.StorageBoxChangeTypeRequest.
func SchemaFromStorageBoxChangeTypeOpts(in StorageBoxChangeTypeOpts) schema.StorageBoxChangeTypeRequest {
	return c.SchemaFromStorageBoxChangeTypeOpts(in)
}

// SchemaFromStorageBoxCreateOpts converts StorageBoxCreateOpts to schema.StorageBoxCreateRequest.
func SchemaFromStorageBoxCreateOpts(in StorageBoxCreateOpts) schema.StorageBoxCreateRequest {
	return c.SchemaFromStorageBoxCreateOpts(in)
}

// SchemaFromStorageBoxEnableSnapshotPlanOpts converts StorageBoxEnableSnapshotPlanOpts to schema.StorageBoxEnableSnapshotPlanRequest.
func SchemaFromStorageBoxEnableSnapshotPlanOpts(in StorageBoxEnableSnapshotPlanOpts) schema.StorageBoxEnableSnapshotPlanRequest {
	return c.SchemaFromStorageBoxEnableSnapshotPlanOpts(in)
}

// SchemaFromStorageBoxResetPasswordOpts converts StorageBoxResetPasswordOpts to schema.StorageBoxResetPasswordRequest.
func SchemaFromStorageBoxResetPasswordOpts(in StorageBoxResetPasswordOpts) schema.StorageBoxResetPasswordRequest {
	return c.SchemaFromStorageBoxResetPasswordOpts(in)
}

// SchemaFromStorageBoxRollbackSnapshotOpts converts StorageBoxRollbackSnapshotOpts to schema.StorageBoxRollbackSnapshotRequest.
func SchemaFromStorageBoxRollbackSnapshotOpts(in StorageBoxRollbackSnapshotOpts) schema.StorageBoxRollbackSnapshotRequest {
	return c.SchemaFromStorageBoxRollbackSnapshotOpts(in)
}

// SchemaFromStorageBoxSnapshot converts *StorageBoxSnapshot to schema.StorageBoxSnapshot.
func SchemaFromStorageBoxSnapshot(in *StorageBoxSnapshot) schema.StorageBoxSnapshot {
	return c.SchemaFromStorageBoxSnapshot(in)
}

// SchemaFromStorageBoxSnapshotCreateOpts converts StorageBoxSnapshotCreateOpts to schema.StorageBoxSnapshotCreateRequest.
func SchemaFromStorageBoxSnapshotCreateOpts(in StorageBoxSnapshotCreateOpts) schema.StorageBoxSnapshotCreateRequest {
	return c.SchemaFromStorageBoxSnapshotCreateOpts(in)
}

// SchemaFromStorageBoxSnapshotUpdateOpts converts StorageBoxSnapshotUpdateOpts to schema.StorageBoxSnapshotUpdateRequest.
func SchemaFromStorageBoxSnapshotUpdateOpts(in StorageBoxSnapshotUpdateOpts) schema.StorageBoxSnapshotUpdateRequest {
	return c.SchemaFromStorageBoxSnapshotUpdateOpts(in)
}

// SchemaFromStorageBoxSubaccount converts *StorageBoxSubaccount to schema.StorageBoxSubaccount.
func SchemaFromStorageBoxSubaccount(in *StorageBoxSubaccount) schema.StorageBoxSubaccount {
	return c.SchemaFromStorageBoxSubaccount(in)
}

// SchemaFromStorageBoxSubaccountChangeHomeDirectoryOpts converts StorageBoxSubaccountChangeHomeDirectoryOpts to schema.StorageBoxSubaccountChangeHomeDirectoryRequest.
func SchemaFromStorageBoxSubaccountChangeHomeDirectoryOpts(in StorageBoxSubaccountChangeHomeDirectoryOpts) schema.StorageBoxSubaccountChangeHomeDirectoryRequest {
	return c.SchemaFromStorageBoxSubaccountChangeHomeDirectoryOpts(in)
}

// SchemaFromStorageBoxSubaccountCreateOpts converts StorageBoxSubaccountCreateOpts to schema.StorageBoxSubaccountCreateRequest.
func SchemaFromStorageBoxSubaccountCreateOpts(in StorageBoxSubaccountCreateOpts) schema.StorageBoxSubaccountCreateRequest {
	return c.SchemaFromStorageBoxSubaccountCreateOpts(in)
}

// SchemaFromStorageBoxSubaccountResetPasswordOpts converts StorageBoxSubaccountResetPasswordOpts to schema.StorageBoxSubaccountResetPasswordRequest.
func SchemaFromStorageBoxSubaccountResetPasswordOpts(in StorageBoxSubaccountResetPasswordOpts) schema.StorageBoxSubaccountResetPasswordRequest {
	return c.SchemaFromStorageBoxSubaccountResetPasswordOpts(in)
}

// SchemaFromStorageBoxSubaccountUpdateAccessSettingsOpts converts StorageBoxSubaccountUpdateAccessSettingsOpts to schema.StorageBoxSubaccountUpdateAccessSettingsRequest.
func SchemaFromStorageBoxSubaccountUpdateAccessSettingsOpts(in StorageBoxSubaccountUpdateAccessSettingsOpts) schema.StorageBoxSubaccountUpdateAccessSettingsRequest {
	return c.SchemaFromStorageBoxSubaccountUpdateAccessSettingsOpts(in)
}

// SchemaFromStorageBoxSubaccountUpdateOpts converts StorageBoxSubaccountUpdateOpts to schema.StorageBoxSubaccountUpdateRequest.
func SchemaFromStorageBoxSubaccountUpdateOpts(in StorageBoxSubaccountUpdateOpts) schema.StorageBoxSubaccountUpdateRequest {
	return c.SchemaFromStorageBoxSubaccountUpdateOpts(in)
}

// SchemaFromStorageBoxType converts *StorageBoxType to schema.StorageBoxType.
func SchemaFromStorageBoxType(in *StorageBoxType) schema.StorageBoxType {
	return c.SchemaFromStorageBoxType(in)
}

// SchemaFromStorageBoxUpdateAccessSettingsOpts converts StorageBoxUpdateAccessSettingsOpts to schema.StorageBoxUpdateAccessSettingsRequest.
func SchemaFromStorageBoxUpdateAccessSettingsOpts(in StorageBoxUpdateAccessSettingsOpts) schema.StorageBoxUpdateAccessSettingsRequest {
	return c.SchemaFromStorageBoxUpdateAccessSettingsOpts(in)
}

// SchemaFromStorageBoxUpdateOpts converts StorageBoxUpdateOpts to schema.StorageBoxUpdateRequest.
func SchemaFromStorageBoxUpdateOpts(in StorageBoxUpdateOpts) schema.StorageBoxUpdateRequest {
	return c.SchemaFromStorageBoxUpdateOpts(in)
}

// SchemaFromVolume converts *Volume to schema.Volume.
func SchemaFromVolume(in *Volume) schema.Volume {
	return c.SchemaFromVolume(in)
}

// SchemaFromZone converts *Zone to schema.Zone.
func SchemaFromZone(in *Zone) schema.Zone {
	return c.SchemaFromZone(in)
}

// SchemaFromZoneChangePrimaryNameserversOpts converts ZoneChangePrimaryNameserversOpts to schema.ZoneChangePrimaryNameserversRequest.
func SchemaFromZoneChangePrimaryNameserversOpts(in ZoneChangePrimaryNameserversOpts) schema.ZoneChangePrimaryNameserversRequest {
	return c.SchemaFromZoneChangePrimaryNameserversOpts(in)
}

// SchemaFromZoneChangeProtectionOpts converts ZoneChangeProtectionOpts to schema.ZoneChangeProtectionRequest.
func SchemaFromZoneChangeProtectionOpts(in ZoneChangeProtectionOpts) schema.ZoneChangeProtectionRequest {
	return c.SchemaFromZoneChangeProtectionOpts(in)
}

// SchemaFromZoneChangeTTLOpts converts ZoneChangeTTLOpts to schema.ZoneChangeTTLRequest.
func SchemaFromZoneChangeTTLOpts(in ZoneChangeTTLOpts) schema.ZoneChangeTTLRequest {
	return c.SchemaFromZoneChangeTTLOpts(in)
}

// SchemaFromZoneCreateOpts converts ZoneCreateOpts to schema.ZoneCreateRequest.
func SchemaFromZoneCreateOpts(in ZoneCreateOpts) schema.ZoneCreateRequest {
	return c.SchemaFromZoneCreateOpts(in)
}

// SchemaFromZoneImportZonefileOpts converts ZoneImportZonefileOpts to schema.ZoneImportZonefileRequest.
func SchemaFromZoneImportZonefileOpts(in ZoneImportZonefileOpts) schema.ZoneImportZonefileRequest {
	return c.SchemaFromZoneImportZonefileOpts(in)
}

// SchemaFromZoneRRSet converts *ZoneRRSet to schema.ZoneRRSet.
func SchemaFromZoneRRSet(in *ZoneRRSet) schema.ZoneRRSet {
	return c.SchemaFromZoneRRSet(in)
}

// SchemaFromZoneRRSetAddRecordsOpts converts ZoneRRSetAddRecordsOpts to schema.ZoneRRSetAddRecordsRequest.
func SchemaFromZoneRRSetAddRecordsOpts(in ZoneRRSetAddRecordsOpts) schema.ZoneRRSetAddRecordsRequest {
	return c.SchemaFromZoneRRSetAddRecordsOpts(in)
}

// SchemaFromZoneRRSetChangeProtectionOpts converts ZoneRRSetChangeProtectionOpts to schema.ZoneRRSetChangeProtectionRequest.
func SchemaFromZoneRRSetChangeProtectionOpts(in ZoneRRSetChangeProtectionOpts) schema.ZoneRRSetChangeProtectionRequest {
	return c.SchemaFromZoneRRSetChangeProtectionOpts(in)
}

// SchemaFromZoneRRSetChangeTTLOpts converts ZoneRRSetChangeTTLOpts to schema.ZoneRRSetChangeTTLRequest.
func SchemaFromZoneRRSetChangeTTLOpts(in ZoneRRSetChangeTTLOpts) schema.ZoneRRSetChangeTTLRequest {
	return c.SchemaFromZoneRRSetChangeTTLOpts(in)
}

// SchemaFromZoneRRSetCreateOpts converts ZoneRRSetCreateOpts to schema.ZoneRRSetCreateRequest.
func SchemaFromZoneRRSetCreateOpts(in ZoneRRSetCreateOpts) schema.ZoneRRSetCreateRequest {
	return c.SchemaFromZoneRRSetCreateOpts(in)
}

// SchemaFromZoneRRSetRemoveRecordsOpts converts ZoneRRSetRemoveRecordsOpts to schema.ZoneRRSetRemoveRecordsRequest.
func SchemaFromZoneRRSetRemoveRecordsOpts(in ZoneRRSetRemoveRecordsOpts) schema.ZoneRRSetRemoveRecordsRequest {
	return c.SchemaFromZoneRRSetRemoveRecordsOpts(in)
}

// SchemaFromZoneRRSetSetRecordsOpts converts ZoneRRSetSetRecordsOpts to schema.ZoneRRSetSetRecordsRequest.
func SchemaFromZoneRRSetSetRecordsOpts(in ZoneRRSetSetRecordsOpts) schema.ZoneRRSetSetRecordsRequest {
	return c.SchemaFromZoneRRSetSetRecordsOpts(in)
}

// SchemaFromZoneRRSetUpdateOpts converts ZoneRRSetUpdateOpts to schema.ZoneRRSetUpdateRequest.
func SchemaFromZoneRRSetUpdateOpts(in ZoneRRSetUpdateOpts) schema.ZoneRRSetUpdateRequest {
	return c.SchemaFromZoneRRSetUpdateOpts(in)
}

// SchemaFromZoneRRSetUpdateRecordsOpts converts ZoneRRSetUpdateRecordsOpts to schema.ZoneRRSetUpdateRecordsRequest.
func SchemaFromZoneRRSetUpdateRecordsOpts(in ZoneRRSetUpdateRecordsOpts) schema.ZoneRRSetUpdateRecordsRequest {
	return c.SchemaFromZoneRRSetUpdateRecordsOpts(in)
}

// SchemaFromZoneUpdateOpts converts ZoneUpdateOpts to schema.ZoneUpdateRequest.
func SchemaFromZoneUpdateOpts(in ZoneUpdateOpts) schema.ZoneUpdateRequest {
	return c.SchemaFromZoneUpdateOpts(in)
}

// ServerFromSchema converts schema.Server to *Server.
func ServerFromSchema(in schema.Server) *Server {
	return c.ServerFromSchema(in)
}

// ServerMetricsFromSchema converts *schema.ServerGetMetricsResponse to *ServerMetrics.
func ServerMetricsFromSchema(in *schema.ServerGetMetricsResponse) (*ServerMetrics, error) {
	return c.ServerMetricsFromSchema(in)
}

// ServerPrivateNetFromSchema converts schema.ServerPrivateNet to ServerPrivateNet.
func ServerPrivateNetFromSchema(in schema.ServerPrivateNet) ServerPrivateNet {
	return c.ServerPrivateNetFromSchema(in)
}

// ServerPublicNetFromSchema converts schema.ServerPublicNet to ServerPublicNet.
func ServerPublicNetFromSchema(in schema.ServerPublicNet) ServerPublicNet {
	return c.ServerPublicNetFromSchema(in)
}

// ServerPublicNetIPv4FromSchema converts schema.ServerPublicNetIPv4 to ServerPublicNetIPv4.
func ServerPublicNetIPv4FromSchema(in schema.ServerPublicNetIPv4) ServerPublicNetIPv4 {
	return c.ServerPublicNetIPv4FromSchema(in)
}

// ServerPublicNetIPv6FromSchema converts schema.ServerPublicNetIPv6 to ServerPublicNetIPv6.
func ServerPublicNetIPv6FromSchema(in schema.ServerPublicNetIPv6) ServerPublicNetIPv6 {
	return c.ServerPublicNetIPv6FromSchema(in)
}

// ServerTypeFromSchema converts schema.ServerType to *ServerType.
func ServerTypeFromSchema(in schema.ServerType) *ServerType {
	return c.ServerTypeFromSchema(in)
}

// StorageBoxFromSchema converts schema.StorageBox to *StorageBox.
func StorageBoxFromSchema(in schema.StorageBox) *StorageBox {
	return c.StorageBoxFromSchema(in)
}

// StorageBoxSnapshotFromSchema converts schema.StorageBoxSnapshot to *StorageBoxSnapshot.
func StorageBoxSnapshotFromSchema(in schema.StorageBoxSnapshot) *StorageBoxSnapshot {
	return c.StorageBoxSnapshotFromSchema(in)
}

// StorageBoxSnapshotPlanFromSchema converts schema.StorageBoxSnapshotPlan to StorageBoxSnapshotPlan.
func StorageBoxSnapshotPlanFromSchema(in schema.StorageBoxSnapshotPlan) StorageBoxSnapshotPlan {
	return c.StorageBoxSnapshotPlanFromSchema(in)
}

// StorageBoxSubaccountFromCreateResponse converts schema.StorageBoxSubaccountCreateResponseSubaccount to *StorageBoxSubaccount.
func StorageBoxSubaccountFromCreateResponse(in schema.StorageBoxSubaccountCreateResponseSubaccount) *StorageBoxSubaccount {
	return c.StorageBoxSubaccountFromCreateResponse(in)
}

// StorageBoxSubaccountFromSchema converts schema.StorageBoxSubaccount to *StorageBoxSubaccount.
func StorageBoxSubaccountFromSchema(in schema.StorageBoxSubaccount) *StorageBoxSubaccount {
	return c.StorageBoxSubaccountFromSchema(in)
}

// StorageBoxTypeFromSchema converts schema.StorageBoxType to *StorageBoxType.
func StorageBoxTypeFromSchema(in schema.StorageBoxType) *StorageBoxType {
	return c.StorageBoxTypeFromSchema(in)
}

// VolumeFromSchema converts schema.Volume to *Volume.
func VolumeFromSchema(in schema.Volume) *Volume {
	return c.VolumeFromSchema(in)
}

// ZoneFromSchema converts schema.Zone to *Zone.
func ZoneFromSchema(in schema.Zone) *Zone {
	return c.ZoneFromSchema(in)
}

// ZoneRRSetFromSchema converts schema.ZoneRRSet to *ZoneRRSet.
func ZoneRRSetFromSchema(in schema.ZoneRRSet) *ZoneRRSet {
	return c.ZoneRRSetFromSchema(in)
}

// ZoneRRSetRecordFromSchema converts schema.ZoneRRSetRecord to *ZoneRRSetRecord.
func ZoneRRSetRecordFromSchema(in schema.ZoneRRSetRecord) *ZoneRRSetRecord {
	return c.ZoneRRSetRecordFromSchema(in)
}

// intISOFromSchema converts schema.ISO to ISO.
func intISOFromSchema(in schema.ISO) ISO {
	return c.intISOFromSchema(in)
}

// intSchemaFromImage converts Image to schema.Image.
func intSchemaFromImage(in Image) schema.Image {
	return c.intSchemaFromImage(in)
}

// schemaFromDatacenterServerTypes converts DatacenterServerTypes to schema.DatacenterServerTypes.
func schemaFromDatacenterServerTypes(in DatacenterServerTypes) schema.DatacenterServerTypes {
	return c.schemaFromDatacenterServerTypes(in)
}

// schemaFromFloatingIPPricing converts FloatingIPPricing to schema.PricingFloatingIP.
func schemaFromFloatingIPPricing(in FloatingIPPricing) schema.PricingFloatingIP {
	return c.schemaFromFloatingIPPricing(in)
}

// schemaFromFloatingIPTypeLocationPricing converts FloatingIPTypeLocationPricing to schema.PricingFloatingIPTypePrice.
func schemaFromFloatingIPTypeLocationPricing(in FloatingIPTypeLocationPricing) schema.PricingFloatingIPTypePrice {
	return c.schemaFromFloatingIPTypeLocationPricing(in)
}

// schemaFromFloatingIPTypePricing converts FloatingIPTypePricing to schema.PricingFloatingIPType.
func schemaFromFloatingIPTypePricing(in FloatingIPTypePricing) schema.PricingFloatingIPType {
	return c.schemaFromFloatingIPTypePricing(in)
}

// schemaFromImagePricing converts ImagePricing to schema.PricingImage.
func schemaFromImagePricing(in ImagePricing) schema.PricingImage {
	return c.schemaFromImagePricing(in)
}

// schemaFromLoadBalancerTypePricing converts LoadBalancerTypePricing to schema.PricingLoadBalancerType.
func schemaFromLoadBalancerTypePricing(in LoadBalancerTypePricing) schema.PricingLoadBalancerType {
	return c.schemaFromLoadBalancerTypePricing(in)
}

// schemaFromPrimaryIPPricing converts PrimaryIPPricing to schema.PricingPrimaryIP.
func schemaFromPrimaryIPPricing(in PrimaryIPPricing) schema.PricingPrimaryIP {
	return c.schemaFromPrimaryIPPricing(in)
}

// schemaFromPrimaryIPTypePricing converts PrimaryIPTypePricing to schema.PricingPrimaryIPTypePrice.
func schemaFromPrimaryIPTypePricing(in PrimaryIPTypePricing) schema.PricingPrimaryIPTypePrice {
	return c.schemaFromPrimaryIPTypePricing(in)
}

// schemaFromServerTypeLocation converts ServerTypeLocation to schema.ServerTypeLocation.
func schemaFromServerTypeLocation(in ServerTypeLocation) schema.ServerTypeLocation {
	return c.schemaFromServerTypeLocation(in)
}

// schemaFromServerTypeLocationPricing converts ServerTypeLocationPricing to schema.PricingServerTypePrice.
func schemaFromServerTypeLocationPricing(in ServerTypeLocationPricing) schema.PricingServerTypePrice {
	return c.schemaFromServerTypeLocationPricing(in)
}

// schemaFromServerTypePricing converts ServerTypePricing to schema.PricingServerType.
func schemaFromServerTypePricing(in ServerTypePricing) schema.PricingServerType {
	return c.schemaFromServerTypePricing(in)
}

// schemaFromTrafficPricing converts TrafficPricing to schema.PricingTraffic.
func schemaFromTrafficPricing(in TrafficPricing) schema.PricingTraffic {
	return c.schemaFromTrafficPricing(in)
}

// schemaFromVolumePricing converts VolumePricing to schema.PricingVolume.
func schemaFromVolumePricing(in VolumePricing) schema.PricingVolume {
	return c.schemaFromVolumePricing(in)
}

// serverTypeLocationFromSchema converts schema.ServerTypeLocation to ServerTypeLocation.
func serverTypeLocationFromSchema(in schema.ServerTypeLocation) ServerTypeLocation {
	return c.serverTypeLocationFromSchema(in)
}

// serverTypeLocationPricingFromSchema converts schema.PricingServerTypePrice to ServerTypeLocationPricing.
func serverTypeLocationPricingFromSchema(in schema.PricingServerTypePrice) ServerTypeLocationPricing {
	return c.serverTypeLocationPricingFromSchema(in)
}
