package text

// colorCSSClassMap contains the equivalent CSS-class for all colors
var colorCSSClassMap = map[Color]string{
	Bold:         "bold",
	Faint:        "faint",
	Italic:       "italic",
	Underline:    "underline",
	BlinkSlow:    "blink-slow",
	BlinkRapid:   "blink-rapid",
	ReverseVideo: "reverse-video",
	Concealed:    "concealed",
	CrossedOut:   "crossed-out",
	FgBlack:      "fg-black",
	FgRed:        "fg-red",
	FgGreen:      "fg-green",
	FgYellow:     "fg-yellow",
	FgBlue:       "fg-blue",
	FgMagenta:    "fg-magenta",
	FgCyan:       "fg-cyan",
	FgWhite:      "fg-white",
	FgHiBlack:    "fg-hi-black",
	FgHiRed:      "fg-hi-red",
	FgHiGreen:    "fg-hi-green",
	FgHiYellow:   "fg-hi-yellow",
	FgHiBlue:     "fg-hi-blue",
	FgHiMagenta:  "fg-hi-magenta",
	FgHiCyan:     "fg-hi-cyan",
	FgHiWhite:    "fg-hi-white",
	BgBlack:      "bg-black",
	BgRed:        "bg-red",
	BgGreen:      "bg-green",
	BgYellow:     "bg-yellow",
	BgBlue:       "bg-blue",
	BgMagenta:    "bg-magenta",
	BgCyan:       "bg-cyan",
	BgWhite:      "bg-white",
	BgHiBlack:    "bg-hi-black",
	BgHiRed:      "bg-hi-red",
	BgHiGreen:    "bg-hi-green",
	BgHiYellow:   "bg-hi-yellow",
	BgHiBlue:     "bg-hi-blue",
	BgHiMagenta:  "bg-hi-magenta",
	BgHiCyan:     "bg-hi-cyan",
	BgHiWhite:    "bg-hi-white",
}
